/*
 * Kernel Debugger Architecture Dependent Support Functions
 *
 * Copyright (C) 1999-2002 Silicon Graphics, Inc.  All Rights Reserved
 * Copyright (C) David Mosberger-Tang <davidm@hpl.hp.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
 * Mountain View, CA  94043, or:
 *
 * http://www.sgi.com
 *
 * For further information regarding this notice, see:
 *
 * http://oss.sgi.com/projects/GenInfo/NoticeExplan
 */

#include <linux/config.h>
#include <linux/string.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/ptrace.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/kdb.h>
#include <linux/kdbprivate.h>

#include <asm/processor.h>
#include <asm/uaccess.h>
#include <asm/rse.h>
#include <asm/delay.h>
#ifdef CONFIG_SMP
#include <asm/hw_irq.h>
#endif

#if defined(CONFIG_SMP)
unsigned long smp_kdb_wait;
#endif

/*
 * kdba_prologue
 *
 *	FIXME:	On ia64 this function is not used yet.  It should be used
 *		so every arch has a common interface.  For the moment,
 *		kdba_bt uses ia64 specific unwind code.
 *
 *	This function analyzes a gcc-generated function prototype
 *	with or without frame pointers to determine the amount of
 *	automatic storage and register save storage is used on the
 *	stack of the target function.  It only counts instructions
 *	that have been executed up to but excluding the current eip.
 * Inputs:
 *	code	Start address of function code to analyze
 *	pc	Current program counter within function
 *	sp	Current stack pointer for function
 *	fp	Current frame pointer for function, may not be valid
 *	ss	Start of stack for current process.
 *	caller	1 if looking for data on the caller frame, 0 for callee.
 * Outputs:
 *	ar	Activation record, all fields may be set.  fp and oldfp
 *		are 0 if they cannot be extracted.  return is 0 if the
 *		code cannot find a valid return address.  args and arg0
 *		are 0 if the number of arguments cannot be safely
 *		calculated.
 * Returns:
 *	1 if prologue is valid, 0 otherwise.  If pc is 0 treat it as a
 *	valid prologue to allow bt on wild branches.
 * Locking:
 *	None.
 * Remarks:
 *	FIXME: design an activation record format that works for all architectures.
 */

int
kdba_prologue(const kdb_symtab_t *symtab, kdb_machreg_t pc, kdb_machreg_t sp,
	      kdb_machreg_t fp, kdb_machreg_t ss, int caller, kdb_ar_t *ar)
{
	return(1);	/*temp*/
}


static int
kdba_itm (int argc, const char **argv, const char **envp, kdb_eframe_t ef)
{
	int diag;
	unsigned long val;

	diag = kdbgetularg(argv[1], &val);
	if (diag)
		return diag;
	kdb_printf("new itm=" kdb_machreg_fmt "\n", val);

	ia64_set_itm(val);
	return 0;
}

static void
kdba_show_intregs(void)
{
	u64 lid, tpr, lrr0, lrr1, itv, pmv, cmcv;

	asm ("mov %0=cr.lid" : "=r"(lid));
	asm ("mov %0=cr.tpr" : "=r"(tpr));
	asm ("mov %0=cr.lrr0" : "=r"(lrr0));
	asm ("mov %0=cr.lrr1" : "=r"(lrr1));
	kdb_printf("lid=" kdb_machreg_fmt ", tpr=" kdb_machreg_fmt ", lrr0=" kdb_machreg_fmt ", llr1=" kdb_machreg_fmt "\n", lid, tpr, lrr0, lrr1);

	asm ("mov %0=cr.itv" : "=r"(itv));
	asm ("mov %0=cr.pmv" : "=r"(pmv));
	asm ("mov %0=cr.cmcv" : "=r"(cmcv));
	kdb_printf("itv=" kdb_machreg_fmt ", pmv=" kdb_machreg_fmt ", cmcv=" kdb_machreg_fmt "\n", itv, pmv, cmcv);

	kdb_printf("irr=0x%016lx,0x%016lx,0x%016lx,0x%016lx\n",
		ia64_get_irr0(), ia64_get_irr1(), ia64_get_irr2(), ia64_get_irr3());

	kdb_printf("itc=0x%016lx, itm=0x%016lx\n", ia64_get_itc(), ia64_get_itm());
}

static int
kdba_sir (int argc, const char **argv, const char **envp, kdb_eframe_t ef)
{
	kdba_show_intregs();

	return 0;
}

void
kdba_installdbreg(kdb_bp_t *bp)
{
	/* FIXME: code this */
}

void
kdba_removedbreg(kdb_bp_t *bp)
{
	/* FIXME: code this */
}

kdb_machreg_t
kdba_getdr(int regnum)
{
	kdb_machreg_t contents = 0;
	unsigned long reg = (unsigned long)regnum;

	__asm__ ("mov %0=ibr[%1]"::"r"(contents),"r"(reg));
//        __asm__ ("mov ibr[%0]=%1"::"r"(dbreg_cond),"r"(value));

	return contents;
}


kdb_machreg_t
kdb_getcr(int regnum)
{
	kdb_machreg_t contents = 0;
	return contents;
}

void
kdba_putdr(int regnum, kdb_machreg_t contents)
{
	/* FIXME: code this */
}

static void
get_fault_regs(fault_regs_t *fr)
{
	fr->ifa = 0 ;
	fr->isr = 0 ;

	__asm__ ("rsm psr.ic;;") ;
	ia64_srlz_d();
	__asm__ ("mov %0=cr.ifa" : "=r"(fr->ifa));
	__asm__ ("mov %0=cr.isr" : "=r"(fr->isr));
	__asm__ ("ssm psr.ic;;") ;
	ia64_srlz_d();
}

static void
show_kernel_regs (void)
{
	unsigned long kr[8];
	int i;

	asm ("mov %0=ar.k0" : "=r"(kr[0])); asm ("mov %0=ar.k1" : "=r"(kr[1]));
	asm ("mov %0=ar.k2" : "=r"(kr[2])); asm ("mov %0=ar.k3" : "=r"(kr[3]));
	asm ("mov %0=ar.k4" : "=r"(kr[4])); asm ("mov %0=ar.k5" : "=r"(kr[5]));
	asm ("mov %0=ar.k6" : "=r"(kr[6])); asm ("mov %0=ar.k7" : "=r"(kr[7]));

	for (i = 0; i < 4; ++i)
		kdb_printf(" kr%d: %016lx  kr%d: %016lx\n", 2*i, kr[2*i], 2*i+1, kr[2*i+1]);
	kdb_printf("\n");
}
static int
change_cur_stack_frame(struct pt_regs *regs, int regno, unsigned long *contents)
{
	unsigned long sof, i, cfm, sp, *bsp;
	struct unw_frame_info info;
	mm_segment_t old_fs;

	unw_init_frame_info(&info, current, kdb_sw[smp_processor_id()]);
	do {
		if (unw_unwind(&info) < 0) {
			kdb_printf("Failed to unwind\n");
			return 0;
		}
		unw_get_sp(&info, &sp);
	} while (sp <= (unsigned long) regs);
	unw_get_bsp(&info, (unsigned long *) &bsp);
	unw_get_cfm(&info, &cfm);

	if (!bsp) {
		kdb_printf("Unable to get Current Stack Frame\n");
		return 0;
	}

	sof = (cfm & 0x7f);

	if(((unsigned long)regno - 32) >= (sof - 2)) return 1;

	old_fs = set_fs(KERNEL_DS);
	{
		for (i = 0; i < (regno - 32); ++i) {
			bsp = ia64_rse_skip_regs(bsp, 1);
		}
		put_user(*contents, bsp);
	}
	set_fs(old_fs);

	return 0 ;
}

static int
show_cur_stack_frame(struct pt_regs *regs, int regno, unsigned long *contents)
{
	unsigned long sof, i, cfm, val, sp, *bsp;
	struct unw_frame_info info;
	mm_segment_t old_fs;

	/* XXX It would be better to simply create a copy of an unw_frame_info structure
	 * that is set up in kdba_main_loop().  That way, we could avoid having to skip
	 * over the first few frames every time...
	 */
	unw_init_frame_info(&info, current, kdb_sw[smp_processor_id()]);
	do {
		if (unw_unwind(&info) < 0) {
			kdb_printf("Failed to unwind\n");
			return 0;
		}
		unw_get_sp(&info, &sp);
	} while (sp <= (unsigned long) regs);
	unw_get_bsp(&info, (unsigned long *) &bsp);
	unw_get_cfm(&info, &cfm);

	if (!bsp) {
		kdb_printf("Unable to display Current Stack Frame\n");
		return 0;
	}

	sof = (cfm & 0x7f);

	if (regno) {
		if ((unsigned) regno - 32 >= sof)
			return 0;
		bsp = ia64_rse_skip_regs(bsp, regno - 32);
		old_fs = set_fs(KERNEL_DS);
		{
			get_user(val, bsp);
		}
		set_fs(old_fs);
		*contents = val;
		return 1;
	}

	old_fs = set_fs(KERNEL_DS);
	{
		for (i = 0; i < sof; ++i) {
			get_user(val, bsp);
			kdb_printf(" r%lu: %016lx ", 32 + i, val);
			if (!((i + 1) % 3))
				kdb_printf("\n");
			bsp = ia64_rse_skip_regs(bsp, 1);
		}
		kdb_printf("\n");
	}
	set_fs(old_fs);

	return 0 ;
}

/*
 * kdba_getregcontents
 *
 *	Return the contents of the register specified by the
 *	input string argument.   Return an error if the string
 *	does not match a machine register.
 *
 *	The following pseudo register names are supported:
 *	   &regs	 - Prints address of exception frame
 *	   kesp		 - Prints kernel stack pointer at time of fault
 * 	   sstk		 - Prints switch stack for ia64
 *	   %<regname>	 - Uses the value of the registers at the
 *			   last time the user process entered kernel
 *			   mode, instead of the registers at the time
 *			   kdb was entered.
 *
 * Parameters:
 *	regname		Pointer to string naming register
 *	regs		Pointer to structure containing registers.
 * Outputs:
 *	*contents	Pointer to unsigned long to recieve register contents
 * Returns:
 *	0		Success
 *	KDB_BADREG	Invalid register name
 * Locking:
 * 	None.
 * Remarks:
 *
 * 	Note that this function is really machine independent.   The kdb
 *	register list is not, however.
 */

static struct kdbregs {
	char   *reg_name;
	size_t	reg_offset;
} kdbreglist[] = {
	{ "psr",	offsetof(struct pt_regs, cr_ipsr) },
	{ "ifs",	offsetof(struct pt_regs, cr_ifs) },
	{ "ip",	offsetof(struct pt_regs, cr_iip) },

	{ "unat", 	offsetof(struct pt_regs, ar_unat) },
	{ "pfs",	offsetof(struct pt_regs, ar_pfs) },
	{ "rsc", 	offsetof(struct pt_regs, ar_rsc) },

	{ "rnat",	offsetof(struct pt_regs, ar_rnat) },
	{ "bsps",	offsetof(struct pt_regs, ar_bspstore) },
	{ "pr",	offsetof(struct pt_regs, pr) },

	{ "ldrs",	offsetof(struct pt_regs, loadrs) },
	{ "ccv",	offsetof(struct pt_regs, ar_ccv) },
	{ "fpsr",	offsetof(struct pt_regs, ar_fpsr) },

	{ "b0",	offsetof(struct pt_regs, b0) },
	{ "b6",	offsetof(struct pt_regs, b6) },
	{ "b7",	offsetof(struct pt_regs, b7) },

	{ "r1",offsetof(struct pt_regs, r1) },
	{ "r2",offsetof(struct pt_regs, r2) },
	{ "r3",offsetof(struct pt_regs, r3) },

	{ "r8",offsetof(struct pt_regs, r8) },
	{ "r9",offsetof(struct pt_regs, r9) },
	{ "r10",offsetof(struct pt_regs, r10) },

	{ "r11",offsetof(struct pt_regs, r11) },
	{ "r12",offsetof(struct pt_regs, r12) },
	{ "r13",offsetof(struct pt_regs, r13) },

	{ "r14",offsetof(struct pt_regs, r14) },
	{ "r15",offsetof(struct pt_regs, r15) },
	{ "r16",offsetof(struct pt_regs, r16) },

	{ "r17",offsetof(struct pt_regs, r17) },
	{ "r18",offsetof(struct pt_regs, r18) },
	{ "r19",offsetof(struct pt_regs, r19) },

	{ "r20",offsetof(struct pt_regs, r20) },
	{ "r21",offsetof(struct pt_regs, r21) },
	{ "r22",offsetof(struct pt_regs, r22) },

	{ "r23",offsetof(struct pt_regs, r23) },
	{ "r24",offsetof(struct pt_regs, r24) },
	{ "r25",offsetof(struct pt_regs, r25) },

	{ "r26",offsetof(struct pt_regs, r26) },
	{ "r27",offsetof(struct pt_regs, r27) },
	{ "r28",offsetof(struct pt_regs, r28) },

	{ "r29",offsetof(struct pt_regs, r29) },
	{ "r30",offsetof(struct pt_regs, r30) },
	{ "r31",offsetof(struct pt_regs, r31) },

};

static const int nkdbreglist = sizeof(kdbreglist) / sizeof(struct kdbregs);

int
kdba_getregcontents(const char *regname, struct pt_regs *regs, unsigned long *contents)
{
	int i;

	if (strcmp(regname, "&regs") == 0) {
		*contents = (unsigned long)regs;
		return 0;
	}

	if (strcmp(regname, "sstk") == 0) {
		*contents = (unsigned long)getprsregs(regs) ;
		return 0;
	}

	if (strcmp(regname, "isr") == 0) {
		fault_regs_t fr ;
		get_fault_regs(&fr) ;
		*contents = fr.isr ;
		return 0 ;
	}

	if (strcmp(regname, "ksp") == 0) {
		*contents = (unsigned long) (regs + 1);
		return 0;
	}

	for (i=0; i<nkdbreglist; i++) {
		if (strstr(kdbreglist[i].reg_name, regname))
			break;
	}

	if (i == nkdbreglist) {
		/* Lets check the rse maybe */
		if (regname[0] == 'r')
			if (show_cur_stack_frame(regs, simple_strtoul(regname+1, 0, 0), contents))
				return 0 ;
		return KDB_BADREG;
	}

	*contents = *(unsigned long *)((unsigned long)regs +
			kdbreglist[i].reg_offset);

	return 0;
}

/*
 * kdba_setregcontents
 *
 *	Set the contents of the register specified by the
 *	input string argument.   Return an error if the string
 *	does not match a machine register.
 *
 *	Supports modification of user-mode registers via
 *	%<register-name>
 *
 * Parameters:
 *	regname		Pointer to string naming register
 *	regs		Pointer to structure containing registers.
 *	contents	Unsigned long containing new register contents
 * Outputs:
 * Returns:
 *	0		Success
 *	KDB_BADREG	Invalid register name
 * Locking:
 * 	None.
 * Remarks:
 */

int
kdba_setregcontents(const char *regname,
		  struct pt_regs *regs,
		  unsigned long contents)
{
	int i, ret = 0, fixed = 0;
	char *endp;
	unsigned long regno;

	if (regname[0] == '%') {
		regname++;
		regs = (struct pt_regs *)
			(current->thread.ksp - sizeof(struct pt_regs));
	}
	/* fixed registers */
	for (i=0; i<nkdbreglist; i++) {
		if (strnicmp(kdbreglist[i].reg_name,
			     regname,
			     strlen(regname)) == 0) {
			fixed = 1;
			break;
		}
	}

	/* stacked registers */
	if (!fixed) {
		regno = (simple_strtoul(&regname[1], &endp, 10));
		if ((regname[0] == 'r') && regno > (unsigned long)31) {
			ret = change_cur_stack_frame(regs, regno, &contents);
			if(!ret) return 0;
		}
	}

	if ((i == nkdbreglist)
	    || (strlen(kdbreglist[i].reg_name) != strlen(regname))
	    || ret) {
		return KDB_BADREG;
	}

	/* just in case of "standard" register */
	*(unsigned long *)((unsigned long)regs + kdbreglist[i].reg_offset) =
		contents;

	return 0;
}

/*
 * kdba_dumpregs
 *
 *	Dump the specified register set to the display.
 *
 * Parameters:
 *	regs		Pointer to structure containing registers.
 *	type		Character string identifying register set to dump
 *	extra		string further identifying register (optional)
 * Outputs:
 * Returns:
 *	0		Success
 * Locking:
 * 	None.
 * Remarks:
 *	This function will dump the general register set if the type
 *	argument is NULL (struct pt_regs).   The alternate register
 *	set types supported by this function:
 *
 *	d 		Debug registers
 *	c		Control registers
 *	u		User registers at most recent entry to kernel
 *	i		Interrupt registers -- same as "irr" command
 * Following not yet implemented:
 *	m		Model Specific Registers (extra defines register #)
 *	r		Memory Type Range Registers (extra defines register)
 *
 *	For now, all registers are covered as follows:
 *
 * 	rd 		- dumps all regs
 *	rd	%isr	- current interrupt status reg, read freshly
 *	rd	s	- valid stacked regs
 * 	rd 	%sstk	- gets switch stack addr. dump memory and search
 *	rd	d	- debug regs, may not be too useful
 *	rd	k	- dump kernel regs
 *
 *	ARs		TB Done
 *	OTHERS		TB Decided ??
 *
 *	Intel wish list
 *	These will be implemented later - Srinivasa
 *
 *      type        action
 *      ----        ------
 *      g           dump all General static registers
 *      s           dump all general Stacked registers
 *      f           dump all Floating Point registers
 *      p           dump all Predicate registers
 *      b           dump all Branch registers
 *      a           dump all Application registers
 *      c           dump all Control registers
 *
 */

int
kdba_dumpregs(struct pt_regs *regs,
	    const char *type,
	    const char *extra)

{
	int i;
	int count = 0;

	if (type
	 && (type[0] == 'u')) {
		type = NULL;
		regs = (struct pt_regs *)
			(current->thread.ksp - sizeof(struct pt_regs));
	}

	if (type == NULL) {
		for (i=0; i<nkdbreglist; i++) {
			kdb_printf("%4s: 0x%16.16lx  ",
				   kdbreglist[i].reg_name,
				   *(unsigned long *)((unsigned long)regs +
						  kdbreglist[i].reg_offset));

			if ((++count % 3) == 0)
				kdb_printf("\n");
		}

		kdb_printf("&regs = %p\n", (void *)regs);

		return 0;
	}

	switch (type[0]) {
	case 'd':
	{
		for(i=0; i<8; i+=2) {
			kdb_printf("idr%d: 0x%16.16lx  idr%d: 0x%16.16lx\n", i,
					kdba_getdr(i), i+1, kdba_getdr(i+1));

		}
		return 0;
	}
	case 'i':
		kdba_show_intregs();
		break;
	case 'k':
		show_kernel_regs();
		break;
	case 'm':
		break;
	case 'r':
		break;

	case 's':
	{
		show_cur_stack_frame(regs, 0, NULL) ;

		return 0 ;
	}

	case '%':
	{
		unsigned long contents ;

		if (!kdba_getregcontents(type+1, regs, &contents))
			kdb_printf("%s = 0x%16.16lx\n", type+1, contents) ;
		else
			kdb_printf("diag: Invalid register %s\n", type+1) ;

		return 0 ;
	}

	default:
		return KDB_BADREG;
	}

	/* NOTREACHED */
	return 0;
}

kdb_machreg_t
kdba_getpc(kdb_eframe_t ef)
{
	return ((struct pt_regs *)ef)->cr_iip + ia64_psr((struct pt_regs *)ef)->ri;
}

int
kdba_setpc(kdb_eframe_t ef, kdb_machreg_t newpc)
{
	((struct pt_regs *)ef)->cr_iip = newpc & ~0xf;
	ia64_psr((struct pt_regs *)ef)->ri = newpc & 0x3;
	KDB_STATE_SET(IP_ADJUSTED);
	return 0;
}

struct kdba_main_loop_data {
	kdb_reason_t reason;
	kdb_reason_t reason2;
	int error;
	kdb_dbtrap_t db_result;
	kdb_eframe_t ef;
	int ret;
};

/*
 * do_kdba_main_loop
 *
 *	Invoked from kdba_main_loop via unw_init_running() after that routine
 *	has pushed a struct switch_stack.
 *
 * Inputs:
 *	info	Unwind information.
 *	data	kdb data passed as void * to unw_init_running.
 * Returns:
 *	none (unw_init_running requires void).  vdata->ret is set to
 *	0	KDB was invoked for an event which it wasn't responsible
 *	1	KDB handled the event for which it was invoked.
 * Outputs:
 *	none
 * Locking:
 *	None.
 * Remarks:
 *	unw_init_running() creates struct switch_stack then struct
 *	unw_frame_info.  We get the address of the info so step over
 *	that to get switch_stack.  Just hope that unw_init_running
 *	does not change its stack usage.
 */

struct switch_stack *kdb_sw[NR_CPUS];

static void
do_kdba_main_loop(struct unw_frame_info *info, void *vdata)
{
	struct kdba_main_loop_data *data = vdata;
	struct switch_stack *prev_sw;
	KDB_DEBUG_STATE("do_kdba_main_loop", data->reason);
	prev_sw = kdb_sw[smp_processor_id()];
	kdb_sw[smp_processor_id()] = (struct switch_stack *)(info+1);
	data->ret = kdb_main_loop(data->reason, data->reason2, data->error, data->db_result, data->ef);
	kdb_sw[smp_processor_id()] = prev_sw;
}

/*
 * kdba_main_loop
 *
 *	Do any architecture specific set up before entering the main kdb loop.
 *	The primary function of this routine is to make all processes look the
 *	same to kdb, kdb must be able to list a process without worrying if the
 *	process is running or blocked, so make all processes look as though they
 *	are blocked.
 *
 * Inputs:
 *	reason		The reason KDB was invoked
 *	error		The hardware-defined error code
 *	error2		kdb's current reason code.  Initially error but can change
 *			acording to kdb state.
 *	db_result	Result from break or debug point.
 *	ef		The exception frame at time of fault/breakpoint.  If reason
 *			is KDB_REASON_SILENT then ef is NULL, otherwise it should
 *			always be valid.
 * Returns:
 *	0	KDB was invoked for an event which it wasn't responsible
 *	1	KDB handled the event for which it was invoked.
 * Outputs:
 *	Builds a switch_stack structure before calling the main loop.
 * Locking:
 *	None.
 * Remarks:
 *	none.
 */

int
kdba_main_loop(kdb_reason_t reason, kdb_reason_t reason2, int error,
	       kdb_dbtrap_t db_result, kdb_eframe_t ef)
{
	struct kdba_main_loop_data data;
	KDB_DEBUG_STATE("kdba_main_loop", reason);
	data.reason = reason;
	data.reason2 = reason2;
	data.error = error;
	data.db_result = db_result;
	data.ef = ef;
	unw_init_running(do_kdba_main_loop, &data);
	return(data.ret);
}

void
kdba_disableint(kdb_intstate_t *state)
{
	int *fp = (int *)state;
	int   flags;

	__save_flags(flags);
	__cli();

	*fp = flags;
}

void
kdba_restoreint(kdb_intstate_t *state)
{
	int flags = *(int *)state;
	__restore_flags(flags);
}

void
kdba_setsinglestep(struct pt_regs *regs)
{
	ia64_psr(regs)->ss = 1;
}

void
kdba_clearsinglestep(struct pt_regs *regs)
{
	ia64_psr(regs)->ss = 0;
}

int
kdba_getcurrentframe(struct pt_regs *regs)
{
	return 0;
}

/*
 * kdba_enable_mce
 *
 *	This function is called once on each CPU to enable machine
 *	check exception handling.
 *
 * Inputs:
 *	None.
 * Outputs:
 *	None.
 * Returns:
 *	None.
 * Locking:
 *	None.
 * Remarks:
 *
 */

void
kdba_enable_mce(void)
{
}

/*
 * kdba_enable_lbr
 *
 *	Enable last branch recording.
 *
 * Parameters:
 *	None.
 * Returns:
 *	None
 * Locking:
 *	None
 * Remarks:
 *	None.
 */

void
kdba_enable_lbr(void)
{
}

/*
 * kdba_disable_lbr
 *
 *	disable last branch recording.
 *
 * Parameters:
 *	None.
 * Returns:
 *	None
 * Locking:
 *	None
 * Remarks:
 *	None.
 */

void
kdba_disable_lbr(void)
{
}

/*
 * kdba_print_lbr
 *
 *	Print last branch and last exception addresses
 *
 * Parameters:
 *	None.
 * Returns:
 *	None
 * Locking:
 *	None
 * Remarks:
 *	None.
 */

void
kdba_print_lbr(void)
{
}

#if defined(CONFIG_SMP)
/*
 * kdba_sendinit
 *
 *      This function implements the 'init' command.
 *
 *      init    [<cpunum>]
 *
 * Inputs:
 *      argc    argument count
 *      argv    argument vector
 *      envp    environment vector
 *      regs    registers at time kdb was entered.
 * Outputs:
 *      None.
 * Returns:
 *      zero for success, a kdb diagnostic if error
 * Locking:
 *      none.
 * Remarks:
 */

int
kdba_sendinit(int argc, const char **argv, const char **envp, struct pt_regs *regs)
{
	unsigned long cpunum;
	int diag;

	if (argc != 1)
		return KDB_ARGCOUNT;

	diag = kdbgetularg(argv[1], &cpunum);
	if (diag)
		return diag;

	if ((cpunum > NR_CPUS) || !test_bit(cpunum, &cpu_online_map))
		return KDB_BADCPUNUM;

	platform_send_ipi(cpunum, 0, IA64_IPI_DM_INIT, 0);
	return 0;
}
#endif /* CONFIG_SMP */

/*
 * kdba_init
 *
 * 	Architecture specific initialization.
 *
 * Parameters:
 *	None.
 * Returns:
 *	None.
 * Locking:
 *	None.
 * Remarks:
 *	None.
 */

void
kdba_init(void)
{
	kdba_enable_lbr();
	kdb_register("irr", kdba_sir, "", "Show interrupt registers", 0);
	kdb_register("itm", kdba_itm, "", "Set new ITM value", 0);
#if defined(CONFIG_SMP)
	kdb_register("init", kdba_sendinit, "", "Send INIT to cpu", 0);
#endif

	return;
}

/*
 * kdba_adjust_ip
 *
 * 	Architecture specific adjustment of instruction pointer before leaving
 *	kdb.
 *
 * Parameters:
 *	reason		The reason KDB was invoked
 *	error		The hardware-defined error code
 *	ef		The exception frame at time of fault/breakpoint.  If reason
 *			is KDB_REASON_SILENT then ef is NULL, otherwise it should
 *			always be valid.
 * Returns:
 *	None.
 * Locking:
 *	None.
 * Remarks:
 *	On IA64, KDB_ENTER() uses break which is a fault, not a trap.  The
 *	instruction pointer must be stepped before leaving kdb, otherwise we
 *	get a loop.
 */

void
kdba_adjust_ip(kdb_reason_t reason, int error, kdb_eframe_t ef)
{
	if (reason == KDB_REASON_ENTER &&
	    !KDB_STATE(IP_ADJUSTED)) {
		struct pt_regs *regs = (struct pt_regs *)ef;
		if (ia64_psr(regs)->ri < 2)
			kdba_setpc(ef, regs->cr_iip + ia64_psr(regs)->ri + 1);
		else
			kdba_setpc(ef, regs->cr_iip + 16);
	}
}
