#ifndef __ASM_SMPBOOT_H
#define __ASM_SMPBOOT_H

#ifndef __ASM_SMP_H
#include "asm/smp.h"
#endif

#define TRAMPOLINE_LOW phys_to_virt(clustered_apic_logical?0x8:0x467)
#define TRAMPOLINE_HIGH phys_to_virt(clustered_apic_logical?0xa:0x469)

#define boot_cpu_apicid (clustered_apic_logical?boot_cpu_logical_apicid:boot_cpu_physical_apicid)

/*
 * To build the logical APIC ID for each CPU we have three cases:
 *  1) Normal flat mode:  use a bitmap of the CPU numbers
 *  2) Logical multi-quad (NUMA-Q):  do nothing, the BIOS has set it up
 *  3) Physical multi-quad (xAPIC clusters):  convert the Intel standard
 *	physical APIC ID to a cluster nibble/cpu bitmap nibble
 */
/***	mps_cpu (index number):   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, ... */
/***  CPUs have xAPIC phys IDs:  00, 01, 02, 03, 10, 11, 12, 13, 20, 21, ... */
/***		its logical ID:  01, 02, 04, 08, 11, 12, 14, 18, 21, 22, ... */
#define physical_to_logical_apicid(phys_apic) ( (1UL << (phys_apic & 0x3)) | (phys_apic & 0xF0U) )

static inline unsigned long apic_ldr_value(unsigned long value)
{
	if (clustered_apic_logical)
		return (value);
	if (clustered_apic_physical)
		return (((value) & ~APIC_LDR_MASK) |
			SET_APIC_LOGICAL_ID(physical_to_logical_apicid(hard_smp_processor_id())));
	return (((value) & ~APIC_LDR_MASK) | SET_APIC_LOGICAL_ID(1UL << smp_processor_id()));
}

/*
 * How to map from phys_cpu_present_map.
 *  1) Normal flat mode:  use the mps_cpu, apicid bitmap
 *  2) Multi-Quad:  only 4 CPUs per cluster, cluster ID in high nibble
 *	use 
 */
#define cpu_present_to_apicid(mps_cpu)	(clustered_apic_logical ? \
 	( ((mps_cpu / 4) * 16) + (1 << (mps_cpu % 4)) ) : \
	 (clustered_apic_physical ? raw_phys_apicid[mps_cpu] : (mps_cpu) ) )
extern unsigned char raw_phys_apicid[NR_CPUS];
#define apicid_to_phys_cpu_present(apicid)	(clustered_apic_mode ? (1UL << (((apicid >> 4) << 2) + (apicid & 0x3))) : (1UL << apicid))

/*
 * Mappings between logical cpu number and logical / physical apicid
 * The first four macros are trivial, but it keeps the abstraction consistent
 */
extern volatile unsigned char logical_apicid_2_cpu[];
extern volatile unsigned char cpu_2_logical_apicid[];
extern volatile unsigned char physical_apicid_2_cpu[];
extern volatile unsigned char cpu_2_physical_apicid[];

#define logical_apicid_to_cpu(apicid) (int)logical_apicid_2_cpu[apicid]
#define cpu_to_logical_apicid(cpu) (int)cpu_2_logical_apicid[cpu]
#define physical_apicid_to_cpu(apicid) (int)physical_apicid_2_cpu[apicid]
#define cpu_to_physical_apicid(cpu) (int)cpu_2_physical_apicid[cpu]
#define boot_apicid_to_cpu(apicid) (int)(clustered_apic_logical ? logical_apicid_2_cpu[apicid] : physical_apicid_2_cpu[apicid])
#define cpu_to_boot_apicid(cpu) (int)(clustered_apic_logical ? cpu_2_logical_apicid[cpu] : cpu_2_physical_apicid[cpu])


#endif
